
/******************************************************************************
 *
 * This file contains the source code for NST-unit invert
 * 
 * This is only an introductionary example.
 *
 *****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include "nst.h"

/*
 *  The following function declarations correspond to the methods
 *  which belong to the "invert" unit. For a detailed description
 *  of these methods and their usage refer to the "NST Programming
 *  Guide"
 * 
 */

extern int exec_invert(unitptr);
extern int adapt_invert(unitptr);
extern int ctrl_invert(int,unitptr);
extern int load_invert(FILE*,unitptr);
extern int save_invert(FILE*,unitptr);
extern int remove_invert(unitptr);


typedef struct var_invert_{	/* local var of unit invert: */
    int iDx;			/* image width */
    int iDy;			/* image height */
} var_invert_t;


/* @MANPAGE invert -- create unit to invert greyscale image  */

unitptr invert(int iDx,		/* width of image */
	       int iDy,		/* height of image */
	       unitptr uHost	/* host unit */
)

/*

RETURN VALUE:

A pointer to the created unit or .NULL in the case of an error.

INTERFACE OF CREATED UNIT:

- .X_in[]:   (input field 0)  input image data
- .X_out[]:  (output field 0) inverted output image

DESCRIPTION:

This is just a toy example which should illustrate the usage
of private folders which could be contributed to NST/NEO.

*/


{
    unitptr uNew;		/* handle to unit to be returned */
    var_invert_t* pVar;		/* handle to local var of uNew */

    if (is_not_unit(uHost)) {
	error_msg(2,"invert",1,NULL); return NULL;
    }

    /* make unit: */
    if ((uNew=primitive_unit(NST_INP, NST_BYTE, iDx*iDy,
			     NST_OUT, NST_BYTE, iDx*iDy,
			     NST_HOST, uHost))==NULL) {

	/* error message */
	error_msg(30,"invert",1,NULL); /* warning: failed to create unit! */
	return NULL;
    }

    /* allocate variables: */
    pVar = CALLOC(1,var_invert_t);

    /* vput variables: */
    pVar->iDx = iDx;
    pVar->iDy = iDy;
    vput(uNew,var_invert_t*,pVar);

    /* set methods: */
    uNew->exec   = exec_invert;
    uNew->adapt  = adapt_invert;
    uNew->remove = remove_invert;
    uNew->load   = load_invert;
    uNew->save   = save_invert;
    uNew->ctrl   = ctrl_invert;

    /* label unit and return: */
    label_unit(uNew,"invert");
    return uNew;

}  /* end invert */

/***************************************************************************
 * 
 *   Methods for unit invert:
 *
 *   Note, that in this toy example only the exec method is filled 
 *   with life. All the other methods were generated by the "mku" tool
 *   and remained unchanged.
 * 
 **************************************************************************/

int exec_invert(unitptr u)
{
    int i, iSize;
    char *pcInp, *pcOut;

    var_invert_t* pVar; /* handle to local var of u */
    vinit(u);
    pVar = vget(u,var_invert_t*);  /* get local var */

    /* exec code here */
    iSize = pVar->iDx * pVar->iDy;

    pcInp = pcNstInp(u,0,0);	/* get pointer to input pin */
    pcOut = pcNstOut(u,0,0);	/* get pointer to output pin */

    for (i=0; i < iSize; i++)
	*pcOut++ = 255 - *pcInp++;

    return OK;
}

int adapt_invert(unitptr u)
{
    var_invert_t* pVar; /* handle to local var of u */
    vinit(u);
    pVar = vget(u,var_invert_t*);  /* get local var */
    /* adapt code here: */
    return OK;
}


int ctrl_invert(int mode, unitptr u)
/* The following values for mode are recognized:
   ...
*/
{
    var_invert_t* pVar; /* handle to local var of u */
    vinit(u);
    pVar = vget(u,var_invert_t*);  /* get local var */
    switch (mode) {
	/* ctrl code here: */
    default: break;
    }
    return OK;
}

int load_invert(FILE *fp, unitptr u)
{
    int ferror;
    if ((ferror=ivread(fp,FFMT,u)) < 0) return ferror;
    /* fvgetcalls here: */
    return OK;
}

int save_invert(FILE *fp, unitptr u)
{
    int ferror;
    if ((ferror=ivwrite(fp,FFMT,u)) < 0) return ferror;
    /* fvputcalls here: */
    return OK;
}

int remove_invert(unitptr u)
{
    /* delete code here: */
    return nst_remove0(u);
}

